import uuid


class Plot:

    def __init__(self, name="Unknown", experiment_id=None):
        self.plot_id = uuid.uuid4()
        self.name = name
        self.experiment_id = experiment_id
        self.figure = None



import json
import uuid

import requests
import pickle
import pandas
from .request import Request
from .mlmodel import MLModel
from .dataframe_toolkit import DataFrameToolkit

class WizataDSAPIClient:

    def __init__(self):
        self.domain = None
        self.user = None
        self.password = None
        self.protocol = "https"

    def __url(self):
        return self.protocol + "://" + self.user + ":" + self.password + "@" + self.domain + "/dsapi/"

    def __header(self):
        return {'Content-Type': 'application/json'}

    def __request_process(self, method, route, content_json=None, return_type="json"):

        # choosing the right request
        if content_json is None:
            response = requests.request(method, self.__url() + route, headers=self.__header())
        else:
            response = requests.post(self.__url() + route, headers=self.__header(), json=content_json)

        # choosing the right return type
        if response.status_code == 200 and return_type == "json":
            return response.json()
        elif response.status_code == 200 and return_type == "pickle":
            return response.content
        elif response.status_code != 200:
            raise RuntimeError(str(response.status_code) + " - " + response.reason)
        else:
            raise RuntimeError("Cannot find the right action to process the request.")

    def get_infos(self):
        return self.__request_process("GET", "get_infos")

    def get_ds_functions(self):
        return self.__request_process("GET", "get_ds_functions")

    def get_models(self):
        json_models = self.__request_process("GET", "get_models")
        ml_models = []
        for json_model in json_models:
            ml_models.append(MLModel(uuid.UUID(json_model["model_id"])))
        return ml_models

    def get_datas(self, query: Request) -> pandas.DataFrame:
        df_bytes = self.__request_process("POST", "get_datas", query.prepare(), "pickle")
        return pickle.loads(df_bytes)

    def simulate(self, ml_model: MLModel, dataframe: pandas.DataFrame) -> pandas.DataFrame:
        simulation_request = {
            "model_id" : str(ml_model.model_id),
            "dataset" : DataFrameToolkit.convert_to_json(dataframe)
        }
        df_json = self.__request_process("POST", "simulate", simulation_request, "json")
        return DataFrameToolkit.convert_to_json(df_json)



import json
from .mlmodel import MLModel


class DSAPIEncoder(json.JSONEncoder):

    def default(self, obj):
        if isinstance(obj, MLModel):
            json_obj = {
                "model_id": str(obj.model_id),
                "experiment_id": str(obj.experiment_id)
            }
            return json_obj
        else:
            type_name = obj.__class__.__name__
            raise TypeError("Unexpected type {0}".format(type_name))


import uuid

from flask import jsonify


class MLModel:

    def __init__(self, model_id=None):

        # Model ID (GUID used to store the entity on backend and the wizatads on the blob storage
        if model_id is None:
            model_id = uuid.uuid4()
        self.model_id = model_id

        # Used to link the wizatads to the experiment who generate it
        self.experiment_id = None

        # The wizatads itself that is stored storage
        self.trained_model = None
        self.scaler = None

        # List of time-series column used as input for the wizatads (index is the timestamp)
        self.input_columns = None

        # Additional columns is generated by the wizatads (None if no additional columns is generated)
        self.output_columns = None

        # Set if the wizatads generate anomalies or not
        self.has_anomalies = False

        # Set the number of labels
        self.label_counts = 0

        # Set if wizatads has target_feat
        self.has_target_feat = False

    def jsonify(self):
        outputs = self.output_columns
        if outputs is None:
            outputs = []

        return jsonify({
            "model_id": str(self.model_id),
            "experiment_id": str(self.experiment_id),
            "input_columns": list(self.input_columns),
            "output_columns": list(outputs),
            "has_anomalies": str(self.has_anomalies),
            "labels_count": str(self.label_counts),
            "has_target_feat": str(self.has_target_feat)
        })

    def load_json(self, json):
        self.label_counts = int(json["labels_count"])
        self.input_columns = json["input_columns"]
        self.output_columns = json["output_columns"]
        self.has_anomalies = json["has_anomalies"]
        if ("has_target_feat" in json.keys()) and (json["has_target_feat"] == 'True'):
            self.has_target_feat = True

    def get_sample_payload(self):
        pl_columns = {"timestamp": "[timestamp]"}
        for hardwareId in self.input_columns:
            pl_columns[hardwareId] = "[" + hardwareId + "]"
        pl_json = {
            "model_id": str(self.model_id),
            "dataset": pl_columns
        }
        return pl_json

